@@ -50,6 +50,15 @@ use strict;
 use warnings;
 
 our $VERSION = '1.05';
+our @EXPORT = qw(
+    deps_concat
+    deps_parse
+    deps_eval_implication
+    deps_iterate
+    deps_compare
+);
+
+use Exporter qw(import);
 
 use Dpkg::Version;
 use Dpkg::Arch qw(get_host_arch get_build_arch);
@@ -57,10 +66,6 @@ use Dpkg::BuildProfiles qw(get_build_profiles);
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
 
-use Exporter qw(import);
-our @EXPORT = qw(deps_concat deps_parse deps_eval_implication
-                deps_iterate deps_compare);
-
 =item deps_eval_implication($rel_p, $v_p, $rel_q, $v_q)
 
 ($rel_p, $v_p) and ($rel_q, $v_q) express two dependencies as (relation,
@@ -154,7 +159,7 @@ sub deps_eval_implication {
     return;
 }
 
-=item my $dep = deps_concat(@dep_list)
+=item $dep = deps_concat(@dep_list)
 
 This function concatenates multiple dependency lines into a single line,
 joining them with ", " if appropriate, and always returning a valid string.
@@ -167,7 +172,7 @@ sub deps_concat {
     return join ', ', grep { defined } @dep_list;
 }
 
-=item my $dep = deps_parse($line, %options)
+=item $dep = deps_parse($line, %options)
 
 This function parses the dependency line and returns an object, either a
 Dpkg::Deps::AND or a Dpkg::Deps::Union. Various options can alter the
@@ -193,7 +198,7 @@ Dpkg::Arch::get_build_arch() to identify the proper architecture.
 =item reduce_arch (defaults to 0)
 
 If set to 1, ignore dependencies that do not concern the current host
-architecture. This implicitely strips off the architecture restriction
+architecture. This implicitly strips off the architecture restriction
 list so that the resulting dependencies are directly applicable to the
 current architecture.
 
@@ -232,6 +237,12 @@ this when parsing non-dependency fields like Conflicts.
 If set to 1, allow build-dep only arch qualifiers, that is “:native”.
 This should be set whenever working with build-deps.
 
+=item tests_dep (defaults to 0)
+
+If set to 1, allow tests-specific package names in dependencies, that is
+"@" and "@builddeps@" (since dpkg 1.18.7). This should be set whenever
+working with dependency fields from F<debian/tests/control>.
+
 =back
 
 =cut
@@ -249,12 +260,21 @@ sub deps_parse {
     $options{reduce_restrictions} //= 0;
     $options{union} //= 0;
     $options{build_dep} //= 0;
+    $options{tests_dep} //= 0;
 
     if ($options{reduce_restrictions}) {
         $options{reduce_arch} = 1;
         $options{reduce_profiles} = 1;
     }
 
+    # Options for Dpkg::Deps::Simple.
+    my %deps_options = (
+        host_arch => $options{host_arch},
+        build_arch => $options{build_arch},
+        build_dep => $options{build_dep},
+        tests_dep => $options{tests_dep},
+    );
+
     # Strip trailing/leading spaces
     $dep_line =~ s/^\s+//;
     $dep_line =~ s/\s+$//;
@@ -263,14 +283,9 @@ sub deps_parse {
     foreach my $dep_and (split(/\s*,\s*/m, $dep_line)) {
         my @or_list = ();
         foreach my $dep_or (split(/\s*\|\s*/m, $dep_and)) {
-	    my $dep_simple = Dpkg::Deps::Simple->new($dep_or, host_arch =>
-	                                             $options{host_arch},
-	                                             build_arch =>
-	                                             $options{build_arch},
-	                                             build_dep =>
-	                                             $options{build_dep});
+	    my $dep_simple = Dpkg::Deps::Simple->new($dep_or, %deps_options);
 	    if (not defined $dep_simple->{package}) {
-		warning(_g("can't parse dependency %s"), $dep_or);
+		warning(g_("can't parse dependency %s"), $dep_or);
 		return;
 	    }
 	    $dep_simple->{arches} = undef if not $options{use_arch};
@@ -302,7 +317,7 @@ sub deps_parse {
     }
     foreach my $dep (@dep_list) {
         if ($options{union} and not $dep->isa('Dpkg::Deps::Simple')) {
-            warning(_g('an union dependency can only contain simple dependencies'));
+            warning(g_('an union dependency can only contain simple dependencies'));
             return;
         }
         $dep_and->add($dep);
@@ -310,7 +325,7 @@ sub deps_parse {
     return $dep_and;
 }
 
-=item my $bool = deps_iterate($deps, $callback_func)
+=item $bool = deps_iterate($deps, $callback_func)
 
 This function visits all elements of the dependency object, calling the
 callback function for each element.
@@ -361,26 +376,31 @@ my %relation_ordering = (
 );
 
 sub deps_compare {
-    my ($a, $b) = @_;
-    return -1 if $a->is_empty();
-    return 1 if $b->is_empty();
-    while ($a->isa('Dpkg::Deps::Multiple')) {
-	return -1 if $a->is_empty();
-	my @deps = $a->get_deps();
-	$a = $deps[0];
+    my ($aref, $bref) = @_;
+
+    my (@as, @bs);
+    deps_iterate($aref, sub { push @as, @_ });
+    deps_iterate($bref, sub { push @bs, @_ });
+
+    while (1) {
+        my ($a, $b) = (shift @as, shift @bs);
+        my $aundef = not defined $a or $a->is_empty();
+        my $bundef = not defined $b or $b->is_empty();
+
+        return  0 if $aundef and $bundef;
+        return -1 if $aundef;
+        return  1 if $bundef;
+
+        my $ar = $a->{relation} // 'undef';
+        my $br = $b->{relation} // 'undef';
+        my $av = $a->{version} // '';
+        my $bv = $b->{version} // '';
+
+        my $res = (($a->{package} cmp $b->{package}) ||
+                   ($relation_ordering{$ar} <=> $relation_ordering{$br}) ||
+                   ($av cmp $bv));
+        return $res if $res != 0;
     }
-    while ($b->isa('Dpkg::Deps::Multiple')) {
-	return 1 if $b->is_empty();
-	my @deps = $b->get_deps();
-	$b = $deps[0];
-    }
-    my $ar = $a->{relation} // 'undef';
-    my $br = $b->{relation} // 'undef';
-    my $av = $a->{version} // '';
-    my $bv = $a->{version} // '';
-    return (($a->{package} cmp $b->{package}) ||
-	    ($relation_ordering{$ar} <=> $relation_ordering{$br}) ||
-	    ($av cmp $bv));
 }
 
 
@@ -405,7 +425,7 @@ dependencies and while trying to simplify them. It represents a set of
 installed packages along with the virtual packages that they might
 provide.
 
-=head2 COMMON FUNCTIONS
+=head2 COMMON METHODS
 
 =over 4
 
@@ -541,7 +561,7 @@ use warnings;
 
 use Carp;
 
-use Dpkg::Arch qw(debarch_is);
+use Dpkg::Arch qw(debarch_is_concerned debarch_list_parse);
 use Dpkg::BuildProfiles qw(parse_build_profiles evaluate_restriction_formula);
 use Dpkg::Version;
 use Dpkg::ErrorHandling;
@@ -559,12 +579,13 @@ sub new {
     $self->{host_arch} = $opts{host_arch} || Dpkg::Arch::get_host_arch();
     $self->{build_arch} = $opts{build_arch} || Dpkg::Arch::get_build_arch();
     $self->{build_dep} = $opts{build_dep} // 0;
+    $self->{tests_dep} = $opts{tests_dep} // 0;
     $self->parse_string($arg) if defined($arg);
     return $self;
 }
 
 sub reset {
-    my ($self) = @_;
+    my $self = shift;
     $self->{package} = undef;
     $self->{relation} = undef;
     $self->{version} = undef;
@@ -582,9 +603,17 @@ sub parse {
 
 sub parse_string {
     my ($self, $dep) = @_;
+
+    my $pkgname_re;
+    if ($self->{tests_dep}) {
+        $pkgname_re = qr/[\@a-zA-Z0-9][\@a-zA-Z0-9+.-]*/;
+    } else {
+        $pkgname_re = qr/[a-zA-Z0-9][a-zA-Z0-9+.-]*/;
+    }
+
     return if not $dep =~
            m{^\s*                           # skip leading whitespace
-              ([a-zA-Z0-9][a-zA-Z0-9+.-]*)  # package name
+              ($pkgname_re)                 # package name
               (?:                           # start of optional part
                 :                           # colon for architecture
                 ([a-zA-Z0-9][a-zA-Z0-9-]*)  # architecture name
@@ -592,18 +621,20 @@ sub parse_string {
               (?:                           # start of optional part
                 \s* \(                      # open parenthesis for version part
                 \s* (<<|<=|=|>=|>>|[<>])    # relation part
-                \s* (.*?)                   # do not attempt to parse version
+                \s* ([^\)\s]+)              # do not attempt to parse version
                 \s* \)                      # closing parenthesis
               )?                            # end of optional part
               (?:                           # start of optional architecture
                 \s* \[                      # open bracket for architecture
-                \s* (.*?)                   # don't parse architectures now
+                \s* ([^\]]+)                # don't parse architectures now
                 \s* \]                      # closing bracket
               )?                            # end of optional architecture
-              (?:                           # start of optional restriction
+              (
+                (?:                         # start of optional restriction
                 \s* <                       # open bracket for restriction
-                \s* (.*)                    # do not parse restrictions now
+                \s* [^>]+                   # do not parse restrictions now
                 \s* >                       # closing bracket
+                )+
               )?                            # end of optional restriction
               \s*$                          # trailing spaces at end
             }x;
@@ -617,7 +648,7 @@ sub parse_string {
 	$self->{version} = Dpkg::Version->new($4);
     }
     if (defined($5)) {
-	$self->{arches} = [ split(/\s+/, $5) ];
+	$self->{arches} = [ debarch_list_parse($5) ];
     }
     if (defined($6)) {
 	$self->{restrictions} = [ parse_build_profiles($6) ];
@@ -711,25 +742,50 @@ sub _arch_is_superset {
     return 1;
 }
 
-# _arch_qualifier_allows_implication($p, $q)
+# _arch_qualifier_implies($p, $q)
 #
 # Returns true if the arch qualifier $p and $q are compatible with the
-# implication $p -> $q, false otherwise. $p/$q can be
-# undef/"any"/"native" or an architecture string.
-sub _arch_qualifier_allows_implication {
+# implication $p -> $q, false otherwise. $p/$q can be undef/"any"/"native"
+# or an architecture string.
+#
+# Because we are handling dependencies in isolation, and the full context
+# of the implications are only known when doing dependency resolution at
+# run-time, we can only assert that they are implied if they are equal.
+sub _arch_qualifier_implies {
     my ($p, $q) = @_;
-    if (defined $p and $p eq 'any') {
-	return 1 if defined $q and $q eq 'any';
-	return 0;
-    } elsif (defined $p and $p eq 'native') {
-	return 1 if defined $q and ($q eq 'any' or $q eq 'native');
-	return 0;
-    } elsif (defined $p) {
-	return 1 if defined $q and ($p eq $q or $q eq 'any');
-	return 0;
+
+    return $p eq $q if defined $p and defined $q;
+    return 1 if not defined $p and not defined $q;
+    return 0;
+}
+
+# _restrictions_imply($p, $q)
+#
+# Returns true if the restrictions $p and $q are compatible with the
+# implication $p -> $q, false otherwise.
+# NOTE: We don't try to be very clever here, so we may conservatively
+# return false when there is an implication.
+sub _restrictions_imply {
+    my ($p, $q) = @_;
+
+    if (not defined $p) {
+       return 1;
+    } elsif (not defined $q) {
+       return 0;
     } else {
-	return 0 if defined $q and $q ne 'any' and $q ne 'native';
-	return 1;
+       # Check whether set difference is empty.
+       my %restr;
+
+       for my $restrlist (@{$q}) {
+           my $reststr = join ' ', sort @{$restrlist};
+           $restr{$reststr} = 1;
+       }
+       for my $restrlist (@{$p}) {
+           my $reststr = join ' ', sort @{$restrlist};
+           delete $restr{$reststr};
+       }
+
+       return keys %restr == 0;
     }
 }
 
@@ -747,8 +803,12 @@ sub implies {
 	return unless _arch_is_superset($self->{arches}, $o->{arches});
 
 	# The arch qualifier must not forbid an implication
-	return unless _arch_qualifier_allows_implication($self->{archqual},
-	                                                 $o->{archqual});
+	return unless _arch_qualifier_implies($self->{archqual},
+	                                      $o->{archqual});
+
+	# Our restrictions must imply the restrictions for o
+	return unless _restrictions_imply($self->{restrictions},
+	                                  $o->{restrictions});
 
 	# If o has no version clause, then our dependency is stronger
 	return 1 if not defined $o->{relation};
@@ -813,28 +873,7 @@ sub arch_is_concerned {
     return 0 if not defined $self->{package}; # Empty dep
     return 1 if not defined $self->{arches};  # Dep without arch spec
 
-    my $seen_arch = 0;
-    foreach my $arch (@{$self->{arches}}) {
-	$arch=lc($arch);
-
-	if ($arch =~ /^!/) {
-	    my $not_arch = $arch;
-	    $not_arch =~ s/^!//;
-
-	    if (debarch_is($host_arch, $not_arch)) {
-		$seen_arch = 0;
-		last;
-	    } else {
-		# !arch includes by default all other arches
-		# unless they also appear in a !otherarch
-		$seen_arch = 1;
-	    }
-	} elsif (debarch_is($host_arch, $arch)) {
-	    $seen_arch = 1;
-	    last;
-	}
-    }
-    return $seen_arch;
+    return debarch_is_concerned($host_arch, @{$self->{arches}});
 }
 
 sub reduce_arch {
@@ -847,7 +886,7 @@ sub reduce_arch {
 }
 
 sub has_arch_restriction {
-    my ($self) = @_;
+    my $self = shift;
     if (defined $self->{arches}) {
 	return $self->{package};
     } else {
@@ -961,7 +1000,7 @@ sub new {
 }
 
 sub reset {
-    my ($self) = @_;
+    my $self = shift;
     $self->{list} = [];
 }
 
@@ -1002,7 +1041,7 @@ sub reduce_arch {
 }
 
 sub has_arch_restriction {
-    my ($self) = @_;
+    my $self = shift;
     my @res;
     foreach my $dep (@{$self->{list}}) {
 	push @res, $dep->has_arch_restriction();
@@ -1263,7 +1302,7 @@ Those methods are not meaningful for this object and always return undef.
 
 =item $union->simplify_deps($facts)
 
-The simplication is done to generate an union of all the relationships.
+The simplification is done to generate an union of all the relationships.
 It uses $simple_dep->merge_union($other_dep) to get its job done.
 
 =back
@@ -1318,7 +1357,7 @@ packages provided (by the set of installed packages).
 
 =over 4
 
-=item my $facts = Dpkg::Deps::KnownFacts->new();
+=item $facts = Dpkg::Deps::KnownFacts->new();
 
 Creates a new object.
 
@@ -1327,8 +1366,6 @@ Creates a new object.
 use strict;
 use warnings;
 
-use Carp;
-
 use Dpkg::Version;
 
 sub new {
@@ -1381,7 +1418,7 @@ sub add_provided_package {
     push @{$self->{virtualpkg}{$pkg}}, [ $by, $rel, $ver ];
 }
 
-=item my ($check, $param) = $facts->check_package($package)
+=item ($check, $param) = $facts->check_package($package)
 
 $check is one when the package is found. For a real package, $param
 contains the version. For a virtual package, $param contains an array
@@ -1399,8 +1436,8 @@ methods where appropriate, but it should not be directly queried.
 sub check_package {
     my ($self, $pkg) = @_;
 
-    carp 'obsolete function, pass Dpkg::Deps::KnownFacts to Dpkg::Deps ' .
-         'methods instead';
+    warnings::warnif('deprecated', 'obsolete function, pass ' .
+                     'Dpkg::Deps::KnownFacts to Dpkg::Deps methods instead');
 
     if (exists $self->{pkg}{$pkg}) {
 	return (1, $self->{pkg}{$pkg}[0]{version});
@@ -1480,11 +1517,15 @@ sub _evaluate_simple_dep {
 
 =head1 CHANGES
 
-=head2 Version 1.05
+=head2 Version 1.06 (dpkg 1.18.7)
+
+New option: Add tests_dep option to Dpkg::Deps::deps_parse().
+
+=head2 Version 1.05 (dpkg 1.17.14)
 
 New function: Dpkg::Deps::deps_iterate().
 
-=head2 Version 1.04
+=head2 Version 1.04 (dpkg 1.17.10)
 
 New options: Add use_profiles, build_profiles, reduce_profiles and
 reduce_restrictions to Dpkg::Deps::deps_parse().
@@ -1492,15 +1533,15 @@ reduce_restrictions to Dpkg::Deps::deps_parse().
 New methods: Add $dep->profile_is_concerned() and $dep->reduce_profiles()
 for all dependency objects.
 
-=head2 Version 1.03
+=head2 Version 1.03 (dpkg 1.17.0)
 
 New option: Add build_arch option to Dpkg::Deps::deps_parse().
 
-=head2 Version 1.02
+=head2 Version 1.02 (dpkg 1.17.0)
 
 New function: Dpkg::Deps::deps_concat()
 
-=head2 Version 1.01
+=head2 Version 1.01 (dpkg 1.16.1)
 
 New method: Add $dep->reset() for all dependency objects.
 
@@ -1513,7 +1554,7 @@ supplementary parameters ($arch and $multiarch).
 Deprecated method: Dpkg::Deps::KnownFacts->check_package() is obsolete,
 it should not have been part of the public API.
 
-=head2 Version 1.00
+=head2 Version 1.00 (dpkg 1.15.6)
 
 Mark the module as public.
 
