@@ -23,7 +23,6 @@
 use strict;
 use warnings;
 
-use Carp;
 use Cwd;
 use File::Temp qw(tempdir);
 use File::Basename;
@@ -33,12 +32,15 @@ use POSIX qw(:sys_wait_h);
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
+use Dpkg::Build::Types;
 use Dpkg::BuildOptions;
 use Dpkg::BuildProfiles qw(set_build_profiles);
+use Dpkg::Conf;
 use Dpkg::Compression;
 use Dpkg::Checksums;
 use Dpkg::Version;
 use Dpkg::Control;
+use Dpkg::Control::Info;
 use Dpkg::Changelog::Parse;
 use Dpkg::Path qw(find_command);
 use Dpkg::IPC;
@@ -59,73 +61,78 @@ sub usage {
 'Usage: %s [<option>...]')
     . "\n\n" . g_(
 'Options:
-  -F             normal full build (binaries and sources, default).
-  -g             source and arch-indep build.
-  -G             source and arch-specific build.
-  -b             binary-only, no source files.
-  -B             binary-only, only arch-specific files.
-  -A             binary-only, only arch-indep files.
-  -S             source-only, no binary files.
-  -nc            do not clean source tree (implies -b).
-  -tc            clean source tree when finished.
-  -D             check build dependencies and conflicts (default).
-  -d             do not check build dependencies and conflicts.
-  -P<profiles>   assume given build profiles as active (comma-separated list).
-  -R<rules>      rules file to execute (default is debian/rules).
-  -T<target>     call debian/rules <target> with the proper environment.
-      --as-root  ensure -T calls the target with root rights.
-  -j[<number>]   jobs to run simultaneously (passed to <rules>), forced mode.
-  -J[<number>]   jobs to run simultaneously (passed to <rules>), opt-in mode.
-  -r<gain-root-command>
-                 command to gain root privileges (default is fakeroot).
-  --check-command=<check-command>
-                 command to check the .changes file (no default).
-  --check-option=<opt>
-                 pass <opt> to <check-command>.
-  --hook-<hook-name>=<hook-command>
-                 set <hook-command> as the hook <hook-name>, known hooks:
-                   init preclean source build binary changes postclean
-                   check sign done
-  -p<sign-command>
-                 command to sign .dsc and/or .changes files
-                   (default is gpg2 or gpg).
-  -k<keyid>      the key to use for signing.
-  -ap            add pause before starting signature process.
-  -us            unsigned source package.
-  -uc            unsigned .changes file.
-      --force-sign
-                 force signing the resulting files.
-      --admindir=<directory>
-                 change the administrative directory.
-  -?, --help     show this help message.
-      --version  show the version.')
+      --build=<type>[,...]    specify the build <type>: full, source, binary,
+                                any, all (default is \'full\').
+  -F                          normal full build (source and binary; default).
+  -g                          source and arch-indep build.
+  -G                          source and arch-specific build.
+  -b                          binary-only, no source files.
+  -B                          binary-only, only arch-specific files.
+  -A                          binary-only, only arch-indep files.
+  -S                          source-only, no binary files.
+  -nc, --no-pre-clean         do not pre clean source tree (implies -b).
+       --pre-clean            pre clean source tree (default).
+  -tc, --post-clean           clean source tree when finished.
+  -D                          check build dependencies and conflicts (default).
+  -d                          do not check build dependencies and conflicts.
+      --[no-]check-builddeps  ditto.
+  -P, --build-profiles=<profiles>
+                              assume comma-separated build profiles as active.
+  -R, --rules-file=<rules>    rules file to execute (default is debian/rules).
+  -T, --rules-target=<target> call debian/rules <target>.
+      --as-root               ensure -T calls the target with root rights.
+  -j, --jobs[=<number>|auto]  jobs to run simultaneously (passed to <rules>),
+                                forced mode.
+  -J, --jobs-try[=<number>|auto]
+                              jobs to run simultaneously (passed to <rules>),
+                                opt-in mode.
+  -r, --root-command=<command>
+                              command to gain root rights (default is fakeroot).
+      --check-command=<command>
+                              command to check the .changes file (no default).
+      --check-option=<opt>    pass <opt> to check <command>.
+      --hook-<name>=<command> set <command> as the hook <name>, known hooks:
+                                init preclean source build binary changes
+                                postclean check sign done
+  -p, --sign-command=<command>
+                              command to sign .dsc and/or .changes files
+                                (default is gpg2 or gpg).
+  -k, --sign-key=<keyid>      the key to use for signing.
+  -ap, --sign-pause           add pause before starting signature process.
+  -us, --unsigned-source      unsigned source package.
+  -uc, --unsigned-changes     unsigned .changes file.
+      --force-sign            force signing the resulting files.
+      --admindir=<directory>  change the administrative directory.
+  -?, --help                  show this help message.
+      --version               show the version.')
     . "\n\n" . g_(
 'Options passed to dpkg-architecture:
-  -a, --host-arch <arch>    set the host Debian architecture.
-  -t, --host-type <type>    set the host GNU system type.
-      --target-arch <arch>  set the target Debian architecture.
-      --target-type <type>  set the target GNU system type.')
+  -a, --host-arch <arch>      set the host Debian architecture.
+  -t, --host-type <type>      set the host GNU system type.
+      --target-arch <arch>    set the target Debian architecture.
+      --target-type <type>    set the target GNU system type.')
     . "\n\n" . g_(
 'Options passed to dpkg-genchanges:
-  -si            source includes orig, if new upstream (default).
-  -sa            source includes orig, always.
-  -sd            source is diff and .dsc only.
-  -v<version>    changes since version <version>.
-  -m<maint>      maintainer for package is <maint>.
-  -e<maint>      maintainer for release is <maint>.
-  -C<descfile>   changes are described in <descfile>.
-      --changes-option=<opt>
-                 pass option <opt> to dpkg-genchanges.')
+  -si                         source includes orig, if new upstream (default).
+  -sa                         source includes orig, always.
+  -sd                         source is diff and .dsc only.
+  -v<version>                 changes since version <version>.
+  -m, --release-by=<maint>    maintainer for this release is <maint>.
+  -e, --build-by=<maint>      maintainer for this build is <maint>.
+  -C<descfile>                changes are described in <descfile>.
+      --changes-option=<opt>  pass option <opt> to dpkg-genchanges.')
     . "\n\n" . g_(
 'Options passed to dpkg-source:
-  -sn            force Debian native source format.
-  -s[sAkurKUR]   see dpkg-source for explanation.
-  -z<level>      compression level to use for source.
-  -Z<compressor> compression to use for source (gz|xz|bzip2|lzma).
-  -i[<regex>]    ignore diffs of files matching regex.
-  -I[<pattern>]  filter out files when building tarballs.
-      --source-option=<opt>
-                 pass option <opt> to dpkg-source.
+  -sn                         force Debian native source format.
+  -s[sAkurKUR]                see dpkg-source for explanation.
+  -z, --compression-level=<level>
+                              compression level to use for source.
+  -Z, --compression=<compressor>
+                              compression to use for source (gz|xz|bzip2|lzma).
+  -i, --diff-ignore[=<regex>] ignore diffs of files matching <regex>.
+  -I, --tar-ignore[=<pattern>]
+                              filter out files when building tarballs.
+      --source-option=<opt>   pass option <opt> to dpkg-source.
 '), $Dpkg::PROGNAME;
 }
 
@@ -168,46 +175,12 @@ my @hook_names = qw(
 my %hook;
 $hook{$_} = undef foreach @hook_names;
 
-use constant BUILD_DEFAULT    => 1;
-use constant BUILD_SOURCE     => 2;
-use constant BUILD_ARCH_DEP   => 4;
-use constant BUILD_ARCH_INDEP => 8;
-use constant BUILD_BINARY     => BUILD_ARCH_DEP | BUILD_ARCH_INDEP;
-use constant BUILD_SOURCE_DEP => BUILD_SOURCE | BUILD_ARCH_DEP;
-use constant BUILD_SOURCE_INDEP => BUILD_SOURCE | BUILD_ARCH_INDEP;
-use constant BUILD_ALL        => BUILD_BINARY | BUILD_SOURCE;
-my $include = BUILD_ALL | BUILD_DEFAULT;
 
-sub build_is_default() { return $include & BUILD_DEFAULT; }
-sub build_sourceonly() { return $include == BUILD_SOURCE; }
-sub build_binaryonly() { return !($include & BUILD_SOURCE); }
-sub build_binaryindep() { return ($include == BUILD_ARCH_INDEP); }
-sub build_opt {
-    if ($include == BUILD_BINARY) {
-        return '-b';
-    } elsif ($include == BUILD_ARCH_DEP) {
-        return '-B';
-    } elsif ($include == BUILD_ARCH_INDEP) {
-        return '-A';
-    } elsif ($include == BUILD_SOURCE) {
-        return '-S';
-    } elsif ($include == BUILD_SOURCE_DEP) {
-        return '-G';
-    } elsif ($include == BUILD_SOURCE_INDEP) {
-        return '-g';
-    } else {
-        croak "build_opt called with include=$include";
-    }
-}
+my $conf = Dpkg::Conf->new();
+$conf->load_config('buildpackage.conf');
 
-sub set_build_type
-{
-    my ($build_type, $build_option) = @_;
-
-    usageerr(g_('cannot combine %s and %s'), build_opt(), $build_option)
-        if not build_is_default and $include != $build_type;
-    $include = $build_type;
-}
+# Inject config options for command-line parser.
+unshift @ARGV, @{$conf};
 
 my $build_opts = Dpkg::BuildOptions->new();
 
@@ -234,13 +207,13 @@ while (@ARGV) {
 	push @source_opts, $1;
     } elsif (/^--changes-option=(.*)$/) {
 	push @changes_opts, $1;
-    } elsif (/^-j(\d*|auto)$/) {
+    } elsif (/^(?:-j|--jobs=)(\d*|auto)$/) {
 	$parallel = $1 || '';
 	$parallel_force = 1;
-    } elsif (/^-J(\d*|auto)$/) {
+    } elsif (/^(?:-J|--jobs-try=)(\d*|auto)$/) {
 	$parallel = $1 || '';
 	$parallel_force = 0;
-    } elsif (/^-r(.*)$/) {
+    } elsif (/^(?:-r|--root-command=)(.*)$/) {
 	my $arg = $1;
 	@rootcommand = split /\s+/, $arg;
     } elsif (/^--check-command=(.*)$/) {
@@ -254,10 +227,12 @@ while (@ARGV) {
 	usageerr(g_('missing hook %s command'), $hook_name)
 	    if not defined $hook_cmd;
 	$hook{$hook_name} = $hook_cmd;
-    } elsif (/^-p(.*)$/) {
+    } elsif (/^(?:-p|--sign-command=)(.*)$/) {
 	$signcommand = $1;
-    } elsif (/^-k(.*)$/) {
+    } elsif (/^(?:-k|--sign-key=)(.*)$/) {
 	$signkey = $1;
+    } elsif (/^--(no-)?check-builddeps$/) {
+	$checkbuilddep = !(defined $1 and $1 eq 'no-');
     } elsif (/^-([dD])$/) {
 	$checkbuilddep = ($1 eq 'D');
     } elsif (/^--ignore-builtin-builddeps$/) {
@@ -267,24 +242,28 @@ while (@ARGV) {
 	warning(g_('-s%s is deprecated; always using gpg style interface'), $1);
     } elsif (/^--force-sign$/) {
 	$signforce = 1;
-    } elsif (/^-us$/) {
+    } elsif (/^-us$/ or /^--unsigned-source$/) {
 	$signsource = 0;
-    } elsif (/^-uc$/) {
+    } elsif (/^-uc$/ or /^--unsigned-changes$/) {
 	$signchanges = 0;
-    } elsif (/^-ap$/) {
+    } elsif (/^-ap$/ or /^--sign-pausa$/) {
 	$signpause = 1;
     } elsif (/^-a$/ or /^--host-arch$/) {
 	$host_arch = shift;
     } elsif (/^-a(.*)$/ or /^--host-arch=(.*)$/) {
 	$host_arch = $1;
-    } elsif (/^-P(.*)$/) {
+    } elsif (/^-P(.*)$/ or /^--build-profiles=(.*)$/) {
 	my $arg = $1;
 	@build_profiles = split /,/, $arg;
     } elsif (/^-s[iad]$/) {
 	push @changes_opts, $_;
-    } elsif (/^-(?:s[insAkurKUR]|[zZ].*|i.*|I.*)$/) {
+    } elsif (/^--(?:compression-level|compression)=.+$/) {
+	push @source_opts, $_;
+    } elsif (/^--(?:diff-ignore|tar-ignore)(?:=.+)?$/) {
+	push @source_opts, $_;
+    } elsif (/^-(?:s[nsAkurKUR]|[zZ].*|i.*|I.*)$/) {
 	push @source_opts, $_; # passed to dpkg-source
-    } elsif (/^-tc$/) {
+    } elsif (/^-tc$/ or /^--post-clean$/) {
 	$cleansource = 1;
     } elsif (/^-t$/ or /^--host-type$/) {
 	$host_type = shift; # Order DOES matter!
@@ -298,46 +277,44 @@ while (@ARGV) {
 	$target_type = shift;
     } elsif (/^--target-type=(.*)$/) {
 	$target_type = $1;
-    } elsif (/^(?:--target|-T)$/) {
+    } elsif (/^(?:--target|--rules-target|-T)$/) {
         $call_target = shift @ARGV;
-    } elsif (/^(?:--target=|-T)(.+)$/) {
+    } elsif (/^(?:--target=|--rules-target=|-T)(.+)$/) {
         $call_target = $1;
     } elsif (/^--as-root$/) {
         $call_target_as_root = 1;
-    } elsif (/^-nc$/) {
+    } elsif (/^--pre-clean$/) {
+	$noclean = 0;
+    } elsif (/^-nc$/ or /^--no-pre-clean$/) {
 	$noclean = 1;
+    } elsif (/^--build=(.*)$/) {
+        set_build_type_from_options($1, $_);
     } elsif (/^-b$/) {
 	set_build_type(BUILD_BINARY, $_);
-	push @changes_opts, '-b';
     } elsif (/^-B$/) {
 	set_build_type(BUILD_ARCH_DEP, $_);
-	push @changes_opts, '-B';
     } elsif (/^-A$/) {
 	set_build_type(BUILD_ARCH_INDEP, $_);
-	push @changes_opts, '-A';
     } elsif (/^-S$/) {
 	set_build_type(BUILD_SOURCE, $_);
-	push @changes_opts, '-S';
     } elsif (/^-G$/) {
-	set_build_type(BUILD_SOURCE_DEP, $_);
-	push @changes_opts, '-G';
+	set_build_type(BUILD_SOURCE | BUILD_ARCH_DEP, $_);
     } elsif (/^-g$/) {
-	set_build_type(BUILD_SOURCE_INDEP, $_);
-	push @changes_opts, '-g';
+	set_build_type(BUILD_SOURCE | BUILD_ARCH_INDEP, $_);
     } elsif (/^-F$/) {
-	set_build_type(BUILD_ALL, $_);
+	set_build_type(BUILD_FULL, $_);
     } elsif (/^-v(.*)$/) {
 	$since = $1;
-    } elsif (/^-m(.*)$/) {
+    } elsif (/^-m(.*)$/ or /^--release-by=(.*)$/) {
 	$maint = $1;
-    } elsif (/^-e(.*)$/) {
+    } elsif (/^-e(.*)$/ or /^--build-by=(.*)$/) {
 	$changedby = $1;
     } elsif (/^-C(.*)$/) {
 	$desc = $1;
     } elsif (m/^-[EW]$/) {
 	# Deprecated option
 	warning(g_('-E and -W are deprecated, they are without effect'));
-    } elsif (/^-R(.*)$/) {
+    } elsif (/^-R(.*)$/ or /^--rules-target=(.*)$/) {
 	my $arg = $1;
 	@debian_rules = split /\s+/, $arg;
     } else {
@@ -345,22 +322,22 @@ while (@ARGV) {
     }
 }
 
-if (($include & BUILD_BINARY) == BUILD_BINARY) {
+if (build_has_all(BUILD_BINARY)) {
     $buildtarget = 'build';
     $binarytarget = 'binary';
-} elsif ($include & BUILD_ARCH_DEP) {
+} elsif (build_has_any(BUILD_ARCH_DEP)) {
     $buildtarget = 'build-arch';
     $binarytarget = 'binary-arch';
-} elsif ($include & BUILD_ARCH_INDEP) {
+} elsif (build_has_any(BUILD_ARCH_INDEP)) {
     $buildtarget = 'build-indep';
     $binarytarget = 'binary-indep';
 }
 
 if ($noclean) {
     # -nc without -b/-B/-A/-S/-F implies -b
-    $include = BUILD_BINARY if build_is_default;
+    set_build_type(BUILD_BINARY) if build_has_any(BUILD_DEFAULT);
     # -nc with -S implies no dependency checks
-    $checkbuilddep = 0 if build_sourceonly;
+    $checkbuilddep = 0 if build_is(BUILD_SOURCE);
 }
 
 if ($< == 0) {
@@ -439,6 +416,8 @@ if ($changedby) {
     $maintainer = mustsetvar($changelog->{maintainer}, g_('source changed by'));
 }
 
+# <https://reproducible-builds.org/specs/source-date-epoch/>
+$ENV{SOURCE_DATE_EPOCH} ||= $changelog->{timestamp};
 
 my @arch_opts;
 push @arch_opts, ('--host-arch', $host_arch) if $host_arch;
@@ -456,12 +435,12 @@ while (<$arch_env>) {
 close $arch_env or subprocerr('dpkg-architecture');
 
 my $arch;
-if (build_sourceonly) {
-    $arch = 'source';
-} elsif (build_binaryindep) {
-    $arch = 'all';
-} else {
+if (build_has_any(BUILD_ARCH_DEP)) {
     $arch = mustsetvar($ENV{DEB_HOST_ARCH}, g_('host architecture'));
+} elsif (build_has_any(BUILD_ARCH_INDEP)) {
+    $arch = 'all';
+} elsif (build_has_any(BUILD_SOURCE)) {
+    $arch = 'source';
 }
 
 my $pv = "${pkg}_$sversion";
@@ -479,7 +458,7 @@ if (not $signcommand) {
     $signchanges = 0;
 }
 
-if ($signsource && build_binaryonly) {
+if ($signsource && build_has_none(BUILD_SOURCE)) {
     $signsource = 0;
 }
 
@@ -503,8 +482,8 @@ unless ($call_target) {
 if ($checkbuilddep) {
     my @checkbuilddep_opts;
 
-    push @checkbuilddep_opts, '-A' if ($include & BUILD_ARCH_DEP) == 0;
-    push @checkbuilddep_opts, '-B' if ($include & BUILD_ARCH_INDEP) == 0;
+    push @checkbuilddep_opts, '-A' if build_has_none(BUILD_ARCH_DEP);
+    push @checkbuilddep_opts, '-B' if build_has_none(BUILD_ARCH_INDEP);
     push @checkbuilddep_opts, '-I' if not $check_builtin_builddep;
     push @checkbuilddep_opts, "--admindir=$admindir" if $admindir;
 
@@ -535,9 +514,9 @@ unless ($noclean) {
     withecho(@rootcommand, @debian_rules, 'clean');
 }
 
-run_hook('source', $include & BUILD_SOURCE);
+run_hook('source', build_has_any(BUILD_SOURCE));
 
-if ($include & BUILD_SOURCE) {
+if (build_has_any(BUILD_SOURCE)) {
     warning(g_('building a source package without cleaning up as you asked; ' .
                'it might contain undesired files')) if $noclean;
     chdir('..') or syserr('chdir ..');
@@ -545,27 +524,13 @@ if ($include & BUILD_SOURCE) {
     chdir($dir) or syserr("chdir $dir");
 }
 
-run_hook('build', $include & BUILD_BINARY);
+run_hook('build', build_has_any(BUILD_BINARY));
 
-if ($buildtarget ne 'build' and scalar(@debian_rules) == 1) {
-    # Verify that build-{arch,indep} are supported. If not, fallback to build.
-    # This is a temporary measure to not break too many packages on a flag day.
-    my $pid = spawn(exec => [ 'make', '-f', @debian_rules, '-qn', $buildtarget ],
-                    from_file => '/dev/null', to_file => '/dev/null',
-                    error_to_file => '/dev/null');
-    my $cmdline = "make -f @debian_rules -qn $buildtarget";
-    wait_child($pid, nocheck => 1, cmdline => $cmdline);
-    my $exitcode = WEXITSTATUS($?);
-    subprocerr($cmdline) unless WIFEXITED($?);
-    if ($exitcode == 2) {
-        warning(g_("%s must be updated to support the 'build-arch' and " .
-                   "'build-indep' targets (at least '%s' seems to be " .
-                   'missing)'), "@debian_rules", $buildtarget);
-        $buildtarget = 'build';
-    }
-}
+# XXX Use some heuristics to decide whether to use build-{arch,indep} targets.
+# This is a temporary measure to not break too many packages on a flag day.
+build_target_fallback();
 
-if ($include & BUILD_BINARY) {
+if (build_has_any(BUILD_BINARY)) {
     withecho(@debian_rules, $buildtarget);
     run_hook('binary', 1);
     withecho(@rootcommand, @debian_rules, $binarytarget);
@@ -573,6 +538,8 @@ if ($include & BUILD_BINARY) {
 
 run_hook('changes', 1);
 
+my $build_types = get_build_options_from_type();
+push @changes_opts, "--build=$build_types" if build_has_none(BUILD_DEFAULT);
 push @changes_opts, "-m$maint" if defined $maint;
 push @changes_opts, "-e$changedby" if defined $changedby;
 push @changes_opts, "-v$since" if defined $since;
@@ -581,7 +548,7 @@ push @changes_opts, "-C$desc" if defined $desc;
 my $chg = "../$pva.changes";
 my $changes = Dpkg::Control->new(type => CTRL_FILE_CHANGES);
 
-print { *STDERR } " dpkg-genchanges @changes_opts >$chg\n";
+printcmd("dpkg-genchanges @changes_opts >$chg");
 
 open my $changes_fh, '-|', 'dpkg-genchanges', @changes_opts
     or subprocerr('dpkg-genchanges');
@@ -599,7 +566,7 @@ chdir('..') or syserr('chdir ..');
 withecho('dpkg-source', @source_opts, '--after-build', $dir);
 chdir($dir) or syserr("chdir $dir");
 
-printf "$Dpkg::PROGNAME: %s\n", describe_build($changes->{'Files'});
+info(describe_build($changes->{'Files'}));
 
 run_hook('check', $check_command);
 
@@ -650,12 +617,12 @@ sub mustsetvar {
     error(g_('unable to determine %s'), $text)
 	unless defined($var);
 
-    print "$Dpkg::PROGNAME: $text $var\n";
+    info("$text $var");
     return $var;
 }
 
 sub withecho {
-    print { *STDERR } " @_\n";
+    printcmd(@_);
     system(@_)
 	and subprocerr("@_");
 }
@@ -666,7 +633,7 @@ sub run_hook {
 
     return if not $cmd;
 
-    print { *STDERR } "$Dpkg::PROGNAME: running hook $name\n";
+    info("running hook $name");
 
     my %hook_vars = (
         '%' => '%',
@@ -696,7 +663,7 @@ sub run_hook {
 sub signfile {
     my $file = shift;
 
-    print { *STDERR } " signfile $file\n";
+    printcmd("signfile $file");
 
     my $signdir = tempdir('dpkg-sign.XXXXXXXX', CLEANUP => 1);
     my $signfile = "$signdir/$file";
@@ -751,3 +718,39 @@ sub describe_build {
         return g_('full upload (original source is included)');
     }
 }
+
+sub build_target_fallback {
+    return if $buildtarget eq 'build';
+    return if scalar @debian_rules != 1;
+
+    # Check if we are building both arch:all and arch:any packages, in which
+    # case we now require working build-indep and build-arch targets.
+    my $pkg_arch = 0;
+    my $ctrl = Dpkg::Control::Info->new();
+
+    foreach my $bin ($ctrl->get_packages()) {
+        if ($bin->{Architecture} eq 'all') {
+            $pkg_arch |= BUILD_ARCH_INDEP;
+        } else {
+            $pkg_arch |= BUILD_ARCH_DEP;
+        }
+    }
+
+    return if $pkg_arch == BUILD_BINARY;
+
+    # Check if the build-{arch,indep} targets are supported. If not, fallback
+    # to build.
+    my $pid = spawn(exec => [ 'make', '-f', @debian_rules, '-qn', $buildtarget ],
+                    from_file => '/dev/null', to_file => '/dev/null',
+                    error_to_file => '/dev/null');
+    my $cmdline = "make -f @debian_rules -qn $buildtarget";
+    wait_child($pid, nocheck => 1, cmdline => $cmdline);
+    my $exitcode = WEXITSTATUS($?);
+    subprocerr($cmdline) unless WIFEXITED($?);
+    if ($exitcode == 2) {
+        warning(g_("%s must be updated to support the 'build-arch' and " .
+                   "'build-indep' targets (at least '%s' seems to be " .
+                   'missing)'), "@debian_rules", $buildtarget);
+        $buildtarget = 'build';
+    }
+}
